#include "Delay_Sub.h"
#include "sLCD.h"
#include "x10.h"
#include "protocol.h"
#include "VRbot.h"
//***************************************************************************
//  HAL Improved.c
//  PIC16F688 MCU @ 8Mhz
//  Uses PIC to send/receive serial commands from VRBot Module.  Pic also
//  sends x10 Bit Commands via PORTA to X10 PL-513 Power Line Interface.
//
//  RC0 is Buzzer Output
//  RC1 is Scanning LED
//  RA0 is Zero Crossing - Pin 1 PL513,  Pull-up
//  RA1 is Transmit to X10 = Pin 4 PL513,  Pin 2 and 3 GND.
//  RC4 Rx, RC5 Tx to VRBot,  PULL-up.
//  sLCD on PORTA, RA4 and RA5.

//***************************************************************************
char dummy, chan, OnOff, mode;
signed char res, res1;
char House;
char DimVal = 0;
char txt[4];
//                           1234567890123456
const char Message1[17]   = "HAL the Butler  ";
const char Message2[17]   = "P16F688   by RCI";
const char Message3[17]   = "HAL is Ready!   ";
const char Message4[17]   = "Scanning...";
const char Message5[17]   = "Transmitting X10";
const char Message12[17]  = "Wait for Command";
const char Message99[17]  = "                ";
//****************************************************************************
//*** Enter Programmed SI and SD Words Here***********************************
//****************************************************************************
const char *WordSD0[] =  {"HAL             ",    // 0
                          "Robot           ",    // 1
                          "                "};   // 2
                          
const char *WordSD1[] =  {"Living Room     ",    // 20
                          "Piano Room      ",    // 21
                          "Kitchen         ",    // 22
                          "Porch           ",    // 23
                          "Bedroom         ",    // 24
                          "Fan             ",    // 25
                          "Bathroom        ",    // 26
                          "Hallway         ",    // 27
                          "Dining Room     ",    // 28
                          "Coffee Pot      ",    // 29
                          "Christmas Tree  ",    // 30
                          "Pond Lights     ",    // 30
                          "                "};   // 25
                          
const char *WordSD2[] =  {"ON              ",    // 0
                          "OFF             ",    // 1
                          "Dim Lights      ",    // 2
                          "                "};   // 3
                          


//****************************************************************************
void Get_SD_Commands(void)
{
        char group = 0;
        while (VRbot_GetGroupCount(group) == 0)
    {
    }
wait_sd:

  VRbot_RecognizeSD(group);           // Check Speaker Dependant Table Group 0
  BytetoStr(group, txt);              // Display Group Value
  sLCD_Outn(2, 14, txt);
//*** Loop Here to Scan for Words within a Group *******
while (1 == 1)
{
   res = VRbot_CheckResult();         // Get Response from VRBOT
   sLCD_Out(2, 1, Message4);          // Display "Scanning"
   if ((res < -1) && (group > 0))     // If command not understood
      {
      Sound_Play(1000, 100);          // Beep 3 times
      Delay100ms();
      Sound_Play(1000, 100);
      Delay100ms();
      Sound_Play(1000, 500);
      Delay100ms();
      group = 0;                      // Go Wait for another Group 0 command
      goto wait_sd;
      }
      if (res < -1)
      goto wait_sd;
//******************************************************
// If Response to "HAL" is Valid,  Display Request for Command
    if(group == 0)                    // If VRBot Group
    {
    if (res >= 0)
      {
      sLCD_Out(2, 1, WordSD0[res]);   // Display "HAL"
      Sound_Play(1000, 500);          // Beep once if command is good
      Delay_ms(500);
      sLCD_Out(2, 1, Message99);
      group = 1;
      goto wait_sd;
      }
    } // End If Group
//******************************************************
// If Room Response is Valid,  Display Recalled Room Data
    if(group == 1)                    // If VRBot Group
    {
                if (res >= 0)
      {
      if (res == 0)                   // Living Room
         {
         chan = 2;                    // x10 Channel Number
         }
      if (res == 1)                   // Piano Room
         {
         chan = 10;                    // x10 Channel Number
         }
      if (res == 2)                   // Kitchen
         {
         chan = 4;                    // x10 Channel Number
         }
      if (res == 3)                   // Porch
         {
         chan = 9;                    // x10 Channel Number
         }
      if (res == 4)                   // Bedroom
         {
         chan = 5;                    // x10 Channel Number
         }
      if (res == 5)                   // Fan
         {
         chan = 8;                    // x10 Channel Number
         }
      if (res == 6)                   // Bathroom
         {
         chan = 6;                    // x10 Channel Number
         }
      if (res == 7)                   // Hallway
         {
         chan = 7;                    // x10 Channel Number
         }
      if (res == 8)                   // Dining Room
         {
         chan = 14;                   // x10 Channel Number
         }
      if (res == 9)                   // Coffee Pot
         {
         chan = 15;                   // x10 Channel Number
         }
      if (res == 10)                  // Christmas Tree
         {
         chan = 3;                    // x10 Channel Number
         }
      if (res == 11)                  // Pond Lights
         {
         chan = 11;                   // x10 Channel Number
         }
      sLCD_Out(2, 1, WordSD1[res]);   // Display Room Selected
      Sound_Play(1000, 500);          // Beep once if command is good
      Delay_ms(500);
      sLCD_Out(2, 1, Message99);
      group = 2;
      goto wait_sd;
      }
    } // End If Group
//******************************************************
// If ON/OFF Command Response is Valid,  Display Recall Light Value
    if(group == 2)                   // If VRBot Group
    {
    if (res >= 0)
      {
      if (res == 0)
         {
         OnOff = 20;
         }
      if (res == 1)
         {
         OnOff = 28;
         }
      sLCD_Out(2, 1, WordSD2[res]);  // Display "ON" or "OFF"
      Sound_Play(1000, 500);         // Beep once if command is good
      Delay_ms(500);
      sLCD_Out(2, 1, Message99);
      group = 0;                     // Reset group for another round
      mode = 3;                      // Set mode for x10 transmit
      goto trans;
      }
    } // End If Group
  PORTC.F1 = 1;               // While waiting for valid command.....
  Delay100ms();               // Pulse LED to show HAL is scanning
  PORTC.F1 = 0;
  Delay100ms();
  } // End While
trans:
      return;                 // Send x10 data to Basic Stamp
} // End Demo_SD
//****************************************************************************
void x10Trans(char House, char Chan, char OnOff, char DimVal)
{
      x10_write(House, Chan);
      x10_write(House, Chan);
      Delay500ms();
      x10_write(House, OnOff);
      x10_write(House, OnOff);
}
//****************************************************************************
void main(void)
{
          //  76543210
  ANSEL =   0b00000000;
  PORTA =   0b00000000;
  PORTC =   0b00000000;
  CMCON0 =  0b00000111;           //  Set all Comparators Off
  OSCCON =  0b01110001;           //  Set Osc @ 8 Mhz
  Uart1_Init(9600);
  Sound_Init(&PORTC, 0);
  TRISA =   0b00001101;
  TRISC =   0b00100000;
  mode = 1;
  House = 'A';
  Init_sDisplay();                // Initialize LCD
  Delay100ms();
  sResetLCD();
  Delay100ms();
  sLCD_Out(1, 1, Message1);       // Display Hal the Butler
  sLCD_Out(2, 1, Message2);       // Display P16F688   by RCI
  Delay2000ms();
// Wait here to Detect VRBot Board
loop:
  if (VRbot_Detect())             // If Detected,  go to Start
     {
     sLCD_Out(1, 1, Message3);    // Display Hal is Ready!
     sLCD_Out(2, 1, Message99);
     Delay2000ms();
     goto start;
     }
goto loop;
start:
//****************************************************************************
while (1 == 1)
  {

                        
switch(mode)
  {
//***************************************************************************
case 1:
      sLCD_Out(1, 1, Message12);      // Display "Wait for Command"
      sLCD_Out(2, 1, Message99);
      Delay1000ms();
      Get_SD_Commands();
      break;
//***************************************************************************
case 2:


break;
//***************************************************************************
case 3:  // Transmit Bit Data to X10 PL-513 Interface

      sLCD_Out(1, 1, Message99);
      sLCD_Out(2, 1, Message5);
      x10Trans(House, chan, OnOff, 0);
      Delay100ms();
      DimVal = 0;
      OnOff = 0;
      chan = 0;
      mode = 1;

break;
//***************************************************************************

        } // End Switch
                        
     } // End While
} // End Main
//****************************************************************************
//****************************************************************************